<?php
namespace Teknisa\PLA\Modules\Cardapio\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use \Setforms2\Shared\ORM\DateTime;

class Pla07500LiberacaoCardapioFiliaisService {
    private $connection;
    private $environment;
    protected $crudService;
    
	public function __construct($connection, $environment, $crudService) {
        $this->connection = $connection;
        $this->environment = $environment;
        $this->crudService = $crudService;
    }

	/**
    * Função responsável por retornar os dados de Mes do Cardápio de acordo com
    * os dados enviados do filtro da tela principal.
    * 
    * @param string $param Código da lista.
    */
    public function findMesCardapio($param){
        $query = "SELECT DISTINCT TO_DATE('01'||TO_CHAR(CARDADIA.DTMESCARD, 'MM/YYYY')) DTORDENA, TO_CHAR(CARDADIA.DTMESCARD, 'MM/YYYY') AS DTMESCARD FROM CARDADIA WHERE EXISTS (SELECT CDLISTA FROM LISTA WHERE NRLISTA = :NRLISTA AND CARDADIA.CDFILIAL = SUBSTR(CDLISTA, 1,4) AND CARDADIA.CDSERVFILI = SUBSTR(CDLISTA, 5,5)) ORDER BY DTORDENA DESC";
        
        $params = array(
            "NRLISTA" => $param
        );
		
		$preparedQuery = $this->connection->prepare($query);
		$preparedQuery->execute($params);
		return $preparedQuery->fetchAll();
    }
    
	/**
    * Função responsável por retornar comensal estimado para os dados filtrados.
    * 
    * @param string|array(String) $cdFilial Código(s) da(s) filial(is) filtrada(s).
    * @param string|array(String) $cdServfili Código(s) do(s) serviços(s) filtrado(s).
    * @param string $dtCardapio Data filtrada.
    * @param string $cdGrupfili Código do agrupamento de filiais filtrado.
    */
    public function returnComensal($dtCardapio, $nrLista){
        $query = "SELECT SE.CDFILIAL, SE.CDSERVFILI
					FROM LISTA L
					     INNER JOIN SERVICO SE ON SE.CDFILIAL = SUBSTR(L.CDLISTA, 1, 4)
					                          AND SE.CDSERVFILI = SUBSTR(L.CDLISTA, 6, 10)
					WHERE L.NRLISTA = :P_NRLISTA
					  AND SE.IDSERVATIV = 'S'
					  AND SE.IDCARDDESCENTR = 'S'
					  AND (SE.QTCOMESTISEG IS NOT NULL
					        OR SE.QTCOMESTITER IS NOT NULL
					        OR SE.QTCOMESTIQUA IS NOT NULL
					        OR SE.QTCOMESTIQUI IS NOT NULL
					        OR SE.QTCOMESTISEX IS NOT NULL
					        OR SE.QTCOMESTISAB IS NOT NULL
					        OR SE.QTCOMESTIDOM IS NOT NULL
					        OR SE.QTCOMESTIFER IS NOT NULL)
					  AND NOT EXISTS (
					        SELECT
					            CDFILIAL, CDSERVFILI
					        FROM
					            CARDADIA CD
					        WHERE
					            CD.CDFILIAL = SE.CDFILIAL
					            AND CD.CDSERVFILI = SE.CDSERVFILI
					            AND DTCARDAPIO =:P_DTCARDAPIO)
					ORDER BY
					    SE.CDFILIAL,
					    SE.CDSERVFILI";
        
        $params = array(
            "P_DTCARDAPIO" => $dtCardapio,
            "P_NRLISTA" => $nrLista
        );
		
		$preparedQuery = $this->connection->prepare($query);
		$preparedQuery->execute($params);
		return $preparedQuery->fetchAll();
    }
    
    /**
    * Função responsável inserir os dados retornados da funçao que retorna dados
    * de comensais estimados.
    * 
    * @param object $comensal Objeto contendo código da filial e do servico do 
    *	cardapio.
    * @param string $cdOperador Código do operador logado.
    * @param string $dtCardapio Data filtrada.
    */
    public function insertCardadia($comensal, $cdOperador, $dtCardapio){
    	$query = "INSERT INTO CARDADIA
				   (CDFILIAL, NRCARDAPIO, CDSERVFILI, CDOPERADOR,
				    DTCARDAPIO, DTMESCARD)
				VALUES
				   (:P_CDFILIAL, :P_NRCARDAPIO, :P_CDSERVFILI, :P_CDOPERADOR,
				    :P_DTCARDAPIO, :P_DTCARDAPIO)";
        
        $params = array(
        	'P_CDOPERADOR' => $cdOperador,
        	'P_DTCARDAPIO' => $dtCardapio
        );
        
        
        foreach($comensal as $key => $com){
			$params['P_CDFILIAL'] = $com['CDFILIAL'];
			$params['P_CDSERVFILI'] = $com['CDSERVFILI'];
			$params['P_NRCARDAPIO'] = $com['NRCARDAPIO'];
			
	        try{
		       	$preparedQuery = $this->connection->prepare($query);
				$preparedQuery->execute($params);
	        } catch (\Exception $e){
        		throw $e;
	        }
        }
    }
    
    /**
    * Função responsável por retornar os dados de Cardápio de acordo com
    * os dados enviados do filtro da tela principal.
    * 
    * @param string|array(String) $cdFilial Código(s) da(s) filial(is) filtrada(s).
    * @param string|array(String) $cdServfili Código(s) do(s) serviços(s) filtrado(s).
    * @param string $cdGrupfili Código do agrupamento de filiais filtrado.
    * @param string $dtCardapio Data filtrada.
    */
    public function findCardapio($nrLista, $dtCardapio, $queryLikeAll = ''){
    	$query = "SELECT 'N' AS IDMARCADO, FL.CDFILIAL,
						( FL.CDFILIAL || ' - ' || FL.NMFILIAL ) NMFILIAL,
						        SE.CDSERVFILI, SE.NMSERVICO, TO_CHAR(CA.DTPREVLIBECARD, 'DD/MM/YYYY DY') DTPREVLIBECARD,
						        CTRL.DTLIBECARDMATR, CTRL.CDOPERLIBECARDMATR,
						        CTRL.DTMESCARDAPROV, OP.NMOPERADOR, CTRL.DTAPROCARDFILI,
						        CA.DTLIMAPROVFILI + NVL(CA.QTDIAPRORAPROV,0) AS DATALIMITEFILI
				    FROM LISTA L
				         INNER JOIN SERVICO SE ON SE.CDFILIAL = SUBSTR(L.CDLISTA, 1, 4)
				                              AND SE.CDSERVFILI = SUBSTR(L.CDLISTA, 6, 10)
				         INNER JOIN FILIAL FL ON FL.CDFILIAL = SE.CDFILIAL
				         INNER JOIN CALENCARD CA ON CA.CDFILIAL = FL.CDFILIAL AND
				                                     CA.DTMESCALENCARD = :P_DTMESCARD
				         LEFT JOIN CTRLAPROVCARD CTRL ON CTRL.CDFILIAL = SE.CDFILIAL
				                                     AND CTRL.CDSERVFILI = SE.CDSERVFILI
				                                     AND CTRL.DTMESCARDAPROV = :P_DTMESCARD
				        
				         LEFT JOIN OPERADOR OP ON OP.CDOPERADOR = CTRL.CDOPERLIBECARDMATR
				    WHERE L.NRLISTA = :P_NRLISTA
				      AND SE.IDSERVATIV = 'S'
				        AND SE.IDCARDDESCENTR = 'S'				    
				        $queryLikeAll
				        ORDER BY
				        FL.CDFILIAL,
				        SE.CDSERVFILI";
		$params = array(
            "P_DTMESCARD" => $dtCardapio,
            "P_NRLISTA" => $nrLista
        );
        
		$preparedQuery = $this->connection->prepare($query);
		$preparedQuery->execute($params);
		return $preparedQuery->fetchAll();
    }
    
    
    /**
    * Função responsável por retornar as datas de liberação e aprovação dos 
    * cardápios passados por parãmetro.
    * 
    * @param array $cardapios Array de registros de cardapios.
    */
    public function getCardapioReleaseDates($cardapios, $dtCardapio){
    	$query = "SELECT CDFILIAL, CDSERVFILI, DTLIBECARDMATR, DTAPROCARDFILI
					  FROM CTRLAPROVCARD
					 WHERE (CDFILIAL, CDSERVFILI) IN (_SERVFILIAL_)
					   AND DTMESCARDAPROV  = :P_DTMESCARDAPROV";
        
        $params = array(
        	"P_DTMESCARDAPROV" => $dtCardapio
        );
		
		$cdFilialArray = array();
		$cdServfiliArray = array();
		
		foreach ($cardapios as $key => $card) {
			$cdFilialArray[] = $card['CDFILIAL'];
			$cdServfiliArray[] = $card['CDSERVFILI'];
		}
		
		$cdFilialArray = array_values($cdFilialArray);
		$cdServfiliArray = array_values($cdServfiliArray);
		
		$arrayParameterString = array();
		
		foreach ($cdFilialArray as $key => $codFil){
			
			array_push($arrayParameterString, "(:P_CDFILIAL_$key, :P_CDSERVFILI_$key)");
			$params["P_CDFILIAL_$key"] = $codFil;
			$params["P_CDSERVFILI_$key"] = $cdServfiliArray[$key];
			
		}
		
		$arrayParameterString = implode(", ", $arrayParameterString);
		$query = str_replace("_SERVFILIAL_", $arrayParameterString, $query);
		
		$preparedQuery = $this->connection->prepare($query);
		$preparedQuery->execute($params);
		return $preparedQuery->fetchAll();
    }
    
    /**
    * Função responsável por realizar liberação dos cardápios que ainda não
    * tem registros na tabela CTRLAPROVCARD.
    * 
    * @params array $card Array de registros de cardápios.
    * @params string $dtCardapio Data filtrada no sistema.
    */
    public function releaseCardInclude($card, $dtCardapio){
    	$query = "INSERT INTO CTRLAPROVCARD
					   (CDFILIAL, CDSERVFILI, CDOPERLIBECARDMATR,
					   DTMESCARDAPROV, DTLIBECARDMATR)
					VALUES
					   (:CDFILIAL, :CDSERVFILI, :CDOPERLIBECARDMATR,
					   :DTMESCARDAPROV, :DTLIBECARDMATR)";
		
		$params = array(
			"CDFILIAL" => $card["CDFILIAL"],
			"CDSERVFILI" => $card["CDSERVFILI"],
			"DTMESCARDAPROV" => $dtCardapio,
			"DTLIBECARDMATR" => date("d/m/Y H:i:s"),
			"CDOPERLIBECARDMATR" => $this->environment->getCdOperador()
		);
		
		
		$preparedQuery = $this->connection->prepare($query);
		$preparedQuery->execute($params);
		
		
		$params['P_IDSITUOPERCARD'] = 'S';
		$params['P_IDTIPOOPERCARD'] = 'L';
		
		$this->recordLogReleaseCard($params);
    }
    
    /**
    * Função responsável por realizar liberação dos cardápios que ja possuem
    * registros na tabela CTRLAPROVCARD.
    * 
    * @params array $card Array de registros de cardápios.
    * @params string $dtCardapio Data filtrada no sistema.
    */
    public function releaseCardUpdate($card, $dtCardapio){
    	$query = "UPDATE CTRLAPROVCARD
					SET    CDOPERLIBECARDMATR = :CDOPERLIBECARDMATR,
					       DTLIBECARDMATR     = :DTLIBECARDMATR
					WHERE  CDFILIAL   = :CDFILIAL   AND
					       CDSERVFILI = :CDSERVFILI AND
					       DTMESCARDAPROV = :DTMESCARDAPROV";
		
		$params = array(
			"CDFILIAL" => $card["CDFILIAL"],
			"CDSERVFILI" => $card["CDSERVFILI"],
			"DTMESCARDAPROV" => $dtCardapio,
			"DTLIBECARDMATR" => date("d/m/Y H:i:s"),
			"CDOPERLIBECARDMATR" => $this->environment->getCdOperador()
		);
		
		$preparedQuery = $this->connection->prepare($query);
		$preparedQuery->execute($params);
		
		
		$params['P_IDSITUOPERCARD'] = 'S';
		$params['P_IDTIPOOPERCARD'] = 'L';
		
		$this->recordLogReleaseCard($params);
    }
    
    /**
    * Função responsável por cancelar a liberação dos cardápios que ja possuem
    * registros na tabela CTRLAPROVCARD.
    * 
    * @params array $card Array de registros de cardápios.
    * @params string $dtCardapio Data filtrada no sistema.
    */
    public function cancelLiberarCard($card, $dtCardapio){
		$query = "UPDATE CTRLAPROVCARD
					SET    CDOPERLIBECARDMATR = :CDOPERLIBECARDMATR,
					       DTLIBECARDMATR     = :DTLIBECARDMATR
					WHERE  CDFILIAL   = :CDFILIAL   AND
					       CDSERVFILI = :CDSERVFILI AND
					       DTMESCARDAPROV = :DTMESCARDAPROV";
		
		$params = array(
			"CDFILIAL" => $card["CDFILIAL"],
			"CDSERVFILI" => $card["CDSERVFILI"],
			"DTMESCARDAPROV" => $dtCardapio,
			"DTLIBECARDMATR" => null,
			"CDOPERLIBECARDMATR" => $this->environment->getCdOperador()
		);
		
		$preparedQuery = $this->connection->prepare($query);
		$preparedQuery->execute($params);
		
		
		$params['P_IDSITUOPERCARD'] = 'N';
		$params['P_IDTIPOOPERCARD'] = 'L';
		$params['DTLIBECARDMATR'] = date("d/m/Y H:i:s");
		
		$this->recordLogReleaseCard($params);
    }
    
    
    /**
    * Função responsável por gravar log de operação depois da liberação ou do 
    * cancelamento de um cardápio.
    * 
    * @param array $params Parametros usados para realizar a liberação.
    */
	private function recordLogReleaseCard($params){
    	$query = "INSERT INTO LOGAPROVCARD
					   (CDFILIAL, CDSERVFILI, CDOPERLOGAPROVCARD, 
					   IDSITUOPERCARD, IDTIPOOPERCARD, DTMESCARDAPROVLOG, 
					   DTLOGAPROVCARD)
					VALUES
					   (:CDFILIAL, :CDSERVFILI, :CDOPERLIBECARDMATR, 
					   :P_IDSITUOPERCARD, :P_IDTIPOOPERCARD,
					   :DTMESCARDAPROV, :DTLIBECARDMATR)";
		
		
		$preparedQuery = $this->connection->prepare($query);
		$preparedQuery->execute($params);
    }
    
    /**
    * Função responsável por substituir parametro único por array de parâmetros
    * em query passada por parâmetro, e definir esses parãmetros no array de 
    * parâmetros passado por parâmetro.
    * 
    * @param &string $query Query onde será definido o array.
    * @param &array $paramsArray Array de parametros onde serão definidos os 
    *	novos parâmetros.
    * @param string $parameterName Nome do parâmetro a ser substituido.
    * @param array $arrayToBind Array de valores para serem passados como 
    *	parâmetro.
    */
	private function setArrayOnQueryParameter(&$query, &$paramsArray, $parameterName, $arrayToBind){
		$arrayParameterString = array();
		
		$arrayToBind = array_values($arrayToBind);
		
		foreach ($arrayToBind as $key => $value){
			
			array_push($arrayParameterString, ':'.$parameterName.'_'.$key);
			$paramsArray[$parameterName.'_'.$key] = $value;
			
		}
		
		$arrayParameterString = implode(', ', $arrayParameterString);
		
		$query = str_replace(':'.$parameterName, $arrayParameterString, $query);
	}
	
	
	/**
    * Função responsável por retornar as filiais e os serviços disponíveis para
    * realizar a pesquisa na findMesCardapio de acordo com o filtro passado pelo
    * usuário.
    * 
    * @param array $nrLista Parametro da lista que contem filial e serviço.
    * @param array $cdServfili parametro de serviço caso tenha.
    */
	public function getParamsServFili($nrLista, $cdServfili){
        $sql = "SELECT DISTINCT SERVICO.CDFILIAL, SERVICO.CDSERVFILI FROM SERVICO, LISTA L WHERE SERVICO.IDSERVATIV = 'S' AND NRLISTA = :NRLISTA AND SERVICO.CDFILIAL = L.CDLISTA AND (INSTR( :CDSERVFILI, SERVICO.CDSERVFILI) <> 0 OR :CDSERVFILI = 'T') AND  SERVICO.IDCARDDESCENTR = 'S' ORDER BY SERVICO.CDFILIAL ASC, SERVICO.CDSERVFILI ASC";

        $params = array(
            "NRLISTA"		=> $nrLista,
            "CDSERVFILI"    => $cdServfili
        );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
    /**
    * Função responsável por retornar as filiais disponíveis para o operador logado.
    * 
    * @param array $cdOperador Código do operador.
    * @param array $nrLista Código da lista.
    */
    public function getFiliaisByOper($cdOperador, $nrLista){
        $sql = "INSERT INTO LISTA (CDLISTA, NRLISTA) (SELECT FI.CDFILIAL, :NRLISTA FROM FILIAL FI, FILIOPER FO WHERE FI.CDFILIAL = FO.CDFILIAL AND FO.CDOPERADOR = :P_OPERADOR)";

        $params = array(
            "NRLISTA" => $nrLista,
            "P_OPERADOR" => $cdOperador
        );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    /**
    * Função responsável por retornar as filiais disponíveis por perador e grupo
    * de filiais.
    * 
    * @param array $cdOperador Código do operador.
    * @param array $cdGrupfili Código do grupo de filiais.
    * @param array $nrLista Código da lista.
    */
    public function getFiliaisByGrupFili($cdOperador, $cdGrupfili, $nrLista) {
        $sql = "INSERT INTO LISTA (CDLISTA, NRLISTA) (SELECT FI.CDFILIAL, :NRLISTA FROM FILIAL FI, FILIOPER FO, FILGRUPFILI FG WHERE FI.CDFILIAL = FO.CDFILIAL AND FO.CDOPERADOR = :P_OPERADOR AND FG.CDFILIAL = FI.CDFILIAL AND :P_CDGRUPFILI = FG.CDGRUPFILI)";

        $params = array(
            "P_OPERADOR"	=> $cdOperador,
            "NRLISTA"	=> $nrLista,
            "P_CDGRUPFILI"	=> $cdGrupfili
        );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function findServico($row) {
        try {
            $query = "SELECT DISTINCT
                            NRORG,
                            CDFILIAL,
                            CDSERVFILI,
                            CDFILIAL||CDSERVFILI AS FILISERV
                        FROM
                            SERVICO
                        WHERE
                            VALOR_NA_LISTA(CDFILIAL, :CDFILIAL) <> 0
                        ORDER BY
                            CDFILIAL,
                            CDSERVFILI";
            
            $params = array(
                "CDFILIAL"   => "_NRLISTA=".$row['NMFILIAL']
            );
            $dataSet = $this->connection->fetchAll($query, $params);
            $resultado = array();
            foreach($dataSet as $servico){
                $resultado[] = $servico['FILISERV'];
            }
            return $resultado;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function findFilial($row) {
        try {
            $query = "SELECT DISTINCT
                             F.NRORG,
                             F.CDFILIAL,
                             F.NMFILIAL,
                             F.IDCNTRRESC,
                             F.IDATIVO,
                             F.CDREGIAO
                        FROM
                             FILIAL        F
                             INNER JOIN FILIOPER      FO ON FO.NRORG = F.NRORG
                                                      AND FO.CDFILIAL = F.CDFILIAL
                             LEFT JOIN FILGRUPFILI   FG ON F.NRORG = FG.NRORG
                                                        AND F.CDFILIAL = FG.CDFILIAL
                        WHERE
                             F.NRORG = :NRORG
                             AND FO.CDOPERADOR = :CDOPERADOR
                             AND ( :CDGRUPFILI = FG.CDGRUPFILI
                                  OR :CDGRUPFILI = 'T' )
                        ORDER BY
                             CDFILIAL";
            $params = array(
                "CDOPERADOR" => $this->environment->getCdOperador(),
                "CDGRUPFILI" => $row['NMGRUPFILI'],
                "NRORG"      => $this->environment->getNrorgTrab()
            );
            $filiais = $this->connection->fetchAll($query, $params);
            $resultado = array();
            foreach($filiais as $filial){
                $resultado[] = $filial['CDFILIAL'];
            }
            return $resultado;
        
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function findOperador($row) { 
    	$conditions = array();
        $dataSet = $this->crudService->getFilteredDataSet("operador", $conditions);
        return array_column($dataSet->getRows(), "CDOPERADOR");
    }
}